// $Id: BaseFileConverter.cc 69 2006-02-11 00:44:22Z jari $


#include "Clone.h"
#include "Data.h"
#include "Parameter.h"

#include <basefile.h>

#include <iostream>


int main(const int argc,const char* argv[]) 
{
	theplu::wenni::Parameter option(argc,argv);
	std::string s=option.input_file();
	std::ifstream is(s.c_str());
	BASE::BaseFileReader bfr(is);
	BASE::BaseFileSection bfs;
	
	do {
		if (!bfr.readSection(bfs)) {
			if (bfr.getError())
				std::cerr << "Error reading section in input: "  << bfr.errText()
									<< std::endl;
			exit(-1);
		}
	}	while (bfs.type != "spots");

	if (option.show()) {
		std::cout << "Available types are:\n";
		std::cout << "\t -assayFields -columns \n\n";
		std::cout << "Available assayFields are:\n";

		std::string af;
		if (bfs.findStringOpt("assayFields",af)) {
			std::vector<std::string> af_vec;
			char delim = '\t';
			BASE::split(af, af_vec, 100, delim);
			for (size_t i=0; i<af_vec.size(); i++)
				std::cout << "\t" << af_vec[i] << "\n";
		}
		else {
			std::cerr << "Error: Cannot find assayFields" << std::endl;
			exit(-1);
		}
		std::cout << std::endl;

		std::cout << "Available columns are:\n";
		std::string c;
		if (bfs.findStringOpt("columns",c)) {
			std::vector<std::string> c_vec;
			char delim = '\t';
			BASE::split(c, c_vec, 100, delim);
			for (size_t i=0; i<c_vec.size()-1; i++)
				std::cout << "\t" << c_vec[i] << "\n";
		}
		else {
			std::cerr << "Error: Cannot find assayFields\n";
			exit(-1);
		}
		std::cout << std::endl;
	}

	// Convert assayFields to GSL-matrices
	std::vector<theplu::wenni::Data> data;
	std::vector<theplu::wenni::Clone> clone;
	for (int i=0; i<option.nof_property(); i++) {
		if (option.type(i)=="-assayFields")
			data.push_back(theplu::wenni::Data(bfs,option.property(i),
																				 option.output_file(i)));
		else if (option.type(i)=="-columns")
			clone.push_back(theplu::wenni::Clone(bfs,option.property(i),
																					 option.output_file(i)));
		else {
			std::cerr << "Error: Unknown type" << option.type(i)
								<< " for " << option.property(i) << "\n";
			exit(-1);
		}
	}

	// Extract data
	if (data.size()) {
		std::vector<std::string> vals(data[0].total_nof_columns());
		while (bfr.readVec(vals))
			for (size_t i=0; i<data.size(); i++) 
				data[i].push_back(vals);
	}

	// Extract columns
	if (clone.size()) {
		std::vector<std::string> vals(clone[0].total_nof_columns());
		while (bfr.readVec(vals))
			for (size_t i=0; i<clone.size(); i++) 
				clone[i].push_back(vals);
	}

	for (size_t i=0; i<data.size(); i++) 
		data[i].write_matrix();
	for (size_t i=0; i<clone.size(); i++) 
		clone[i].write();

	return 0;
}
