// $Id: NNI.cc 69 2006-02-11 00:44:22Z jari $

#include <c++_tools/utility/NNI.h>

#include <c++_tools/utility/stl_utility.h>

#include <algorithm>
#include <cmath>
#include <fstream>


namespace theplu {
namespace utility {

	using namespace std;

	// For a discussion and motivation for various algorithm
	// implementations here see the paper cited in the class definition
	// documentation.
	NNI::NNI(const gslapi::matrix& matrix,const gslapi::matrix& weight,
					 const u_int neighbours)
		: data_(matrix), imputed_data_(matrix), neighbours_(neighbours),
			weight_(weight)
	{
	}


  // d_{ij}^2=\frac {\sum_{k=1,C} w_{ik} w_{jk} (x_{ik}-x_{jk})^2 }
  //                {\sum_{k=l,C} w_{ik} w_{jk} }
	// where C is the number of columns
	vector<pair<u_int,double> > NNI::calculate_distances(const u_int row) const
	{
		vector<pair<u_int,double> > distance;
		for (size_t i=0; i<data_.rows(); i++)
			if (i!=row) {
				double contribs=0;
				pair<u_int,double> this_distance(i,0.0);
				for (size_t j=0; j<data_.columns(); j++)
					// 0 contribution for missing values
					if (weight_(i,j) && weight_(row,j)) {
						double weight_factor=weight_(i,j)*weight_(row,j);
						this_distance.second+=( weight_factor *
																		(data_(i,j)-data_(row,j)) *
																		(data_(i,j)-data_(row,j)) );
						contribs+=weight_factor;
					}
				if (contribs) {	// ignore lines without any contributions
					this_distance.second=sqrt(this_distance.second/contribs);
					distance.push_back(this_distance);
				}
			}
		return distance;
	}



	// Contributing nearest neighbours are added up to the user set
	// number, and neighbours are disqualified if their element (column)
	// weight is zero
	vector<u_int> 
	NNI::nearest_neighbours(const u_int column,
													const vector<pair<u_int,double> >& d) const
	{
		vector<u_int> index;
		double contribs=0;
		for (u_int i=0; ((i<d.size()) &&
										 (contribs+=weight_(d[i].first,column))<=neighbours_); i++)
			if (weight_(d[i].first,column))
				index.push_back(i);
		return index;
	}


}} // of namespace utility and namespace theplu
