// $Id: WeNNI.cc 69 2006-02-11 00:44:22Z jari $

#include <c++_tools/utility/WeNNI.h>

#include <c++_tools/utility/stl_utility.h>

#include <algorithm>
#include <cmath>
#include <fstream>

namespace theplu {
namespace utility {


	WeNNI::WeNNI(const gslapi::matrix& matrix,const gslapi::matrix& flag,
							 const u_int neighbours)
		: NNI(matrix,flag,neighbours), imputed_data_raw_(matrix)
	{
		//estimate();
	}



	// \hat{x_{ij}}=\frac{ \sum_{k=1,N} \frac{w_{kj}*x_{kj}}{d_{ki}} }
	//                   { \sum_{k=1,N} \frac{w_{kj}       }{d_{ki}} }
	// where N is defined in the paper cited in the NNI class definition
	// documentation.
	u_int WeNNI::estimate(void)
	{
		using namespace std;
		for (unsigned int i=0; i<data_.rows(); i++) {
			// Jari, avoid copying in next line
			vector<pair<u_int,double> > distance=calculate_distances(i);
 			sort(distance.begin(),distance.end(),
								pair_value_compare<u_int,double>());
			bool row_imputed=true;
			for (unsigned int j=0; j<data_.columns(); j++) {
				vector<u_int> knn=nearest_neighbours(j,distance);
				double new_value=0.0;
				double norm=0.0;
				for (vector<u_int>::const_iterator k=knn.begin(); k!=knn.end(); k++) {
					// Jari, a small number needed here, use something standardized.
					// Avoid division with zero (perfect match vectors)
					double d=(distance[*k].second ? distance[*k].second : 1e-10);
					new_value+=(weight_(distance[*k].first,j) *
											data_(distance[*k].first,j)/d);
					norm+=weight_(distance[*k].first,j)/d;
				}
				// No impute if no contributions from neighbours.
				if (norm){
					imputed_data_raw_(i,j) = new_value/norm;
					imputed_data_(i,j)=
						weight_(i,j)*data_(i,j) + (1-weight_(i,j))* imputed_data_raw_(i,j);
				}
				else
					row_imputed=false;
			}
			if (!row_imputed)
				not_imputed_.push_back(i);
		}
		return not_imputed_.size();
	}


}} // of namespace utility and namespace theplu
