// $Id: weight.h 74 2006-02-15 00:36:50Z jari $

#ifndef _thep_lu_se_wenni_weight_ 
#define _thep_lu_se_wenni_weight_ 

#include <c++_tools/gslapi/matrix.h>

namespace theplu {
namespace wenni {  
namespace weight {

	///
	/// The snr is defined using the foreground, background, and
	/// standard deviation of the background as \f$ (fg-bg)/bgstd \f$ if
	/// \a fg is larger than \a bg, 0 otherwise.
	///
	inline double get_snr(const double fg, const double bg, const double bgstd)
	{ return ((fg>bg) ? (fg-bg)/bgstd : 0.0); }

	///
	/// @return A non-zero weight if \a snr is a non-zero positive
	/// number, zero otherwise.
	///
	inline const double weight_SNR(const double snr, const double beta=0.6)
	{ return (snr>0) ? 1.0/(1.0+beta*beta/snr/snr) : 0.0;}

	///
	/// @return The weight matrix from a \a snr matrix. Indivividual
	/// weight elements are computed with function weight_SNR(const
	/// double snr, const double beta=0.6).
	///
	inline gslapi::matrix weight_SNR(const theplu::gslapi::matrix& snr, 
																	 const double beta=0.6)
	{
		gslapi::matrix w(snr.rows(), snr.columns());
		for (size_t i=0; i<w.rows(); i++) 
			for (size_t j=0; j<w.columns(); j++) 
				w(i,j)=weight_SNR(snr(i,j),beta);
		return w;
	}

	///
	/// @return A non-zero weight if \a snr1 and \a snr2 are both
	/// non-zero positive numbers, zero otherwise.
	///
	inline double weight_SNR2(const double snr1, const double snr2,
														const double beta=0.6)
	{ return (((snr1>0) && (snr2>0)) ?
						1.0/(1+beta*beta/snr1/snr1+beta*beta/snr2/snr2) : 0.0); }

}}} // of namespace weight, wenni, and theplu

#endif

