// $Id: wenni_test.cc 86 2006-04-04 07:00:40Z jari $

#include "c++_tools/gslapi/matrix.h"

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string>
#include <unistd.h>

bool check_default_results(void);
bool check_knni_results(void);
bool check_raw_results(void);
bool check_wenni_snr_results(void);
bool cleanup(void);
bool compare(const std::string&, const std::string&, double error_bound=1e-20);
bool diff(const std::string&, const std::string&);
bool rm(const std::string& filename);


int main(const int argc,const char* argv[])
{  
	bool teststatus=false;

	std::string system_call="./wenni.pl --datatype raw --nodelete";
	system_call+=" < ../data/basefile_in.data > basefile_out.data";
	if (system(system_call.c_str())) {
		teststatus|=true;
		std::cerr << "ERROR: system call failed:\n\t" << system_call << std::endl;
	}
	teststatus|=check_raw_results();

	system_call= "./wenni.pl --nodelete";
	system_call+=" < ../data/basefile_in.data > basefile_out.data";
	if (system(system_call.c_str())) {
		teststatus|=true;
		std::cerr << "ERROR: system call failed:\n\t" << system_call << std::endl;
	}
	teststatus|=check_default_results();

	system_call= "./nni -data wenni_logratio.data -neighbours 10";
	system_call+=" -weight ../data/wenni_snr.data -weight_is_snr";
	system_call+=" > wenni_imputed.data";
	if (system(system_call.c_str())) {
		teststatus|=true;
		std::cerr << "ERROR: system call failed:\n\t" << system_call << std::endl;
	}
	teststatus|=check_wenni_snr_results();

	system_call= "./nni -data wenni_logratio.data -neighbours 10";
	system_call+=" -nni_algorithm kNNI -weight wenni_weight.data";
	system_call+=" -weight_cutoff 0.5 > knni_imputed.data";
	if (system(system_call.c_str())) {
		teststatus|=true;
		std::cerr << "ERROR: system call failed:\n\t" << system_call << std::endl;
	}
	teststatus|=check_knni_results();

	teststatus|=cleanup();

	return (teststatus ? -1 : 0);
}


bool check_default_results(void)
{
	bool teststatus=false;
	teststatus|=diff("BASEfile.data","../data/basefile_in.data");
	teststatus|=diff("basefile_out.data","../data/basefile_out.data");
	teststatus|=compare("wenni_BCh1SD.data","../data/wenni_BCh1SD.data");
	teststatus|=compare("wenni_BCh2SD.data","../data/wenni_BCh2SD.data");
	teststatus|=compare("wenni_imputed.data","../data/wenni_imputed.data");
	teststatus|=compare("wenni_intensity1.data","../data/wenni_intensity1.data");
	teststatus|=compare("wenni_intensity2.data","../data/wenni_intensity2.data");
	teststatus|=compare("wenni_logratio.data","../data/wenni_logratio.data");
	teststatus|=compare("wenni_weight.data","../data/wenni_weight.data");
	if (teststatus)
		std::cerr << "ERROR: check_default_results failed." << std::endl;
	return teststatus;
}


bool check_knni_results(void)
{
	bool teststatus=false;
	teststatus|=diff("knni_imputed.data","../data/knni_imputed.data");
	if (teststatus)
		std::cerr << "ERROR: check_knni_results failed." << std::endl;
	return teststatus;
}


bool check_raw_results(void)
{
	bool teststatus=false;
	teststatus|=diff("BASEfile.data","../data/basefile_in.data");
	teststatus|=diff("basefile_out.data","../data/basefile_out.data");
	teststatus|=compare("wenni_BCh1Mean.data","../data/wenni_BCh1Mean.data");
	teststatus|=compare("wenni_BCh1SD.data","../data/wenni_BCh1SD.data");
	teststatus|=compare("wenni_BCh2Mean.data","../data/wenni_BCh2Mean.data");
	teststatus|=compare("wenni_BCh2SD.data","../data/wenni_BCh2SD.data");
	teststatus|=compare("wenni_FCh1Mean.data","../data/wenni_FCh1Mean.data");
	teststatus|=compare("wenni_FCh2Mean.data","../data/wenni_FCh2Mean.data");
	teststatus|=compare("wenni_imputed.data","../data/wenni_imputed.data");
	teststatus|=compare("wenni_logratio.data","../data/wenni_logratio.data");
	teststatus|=compare("wenni_weight.data","../data/wenni_weight.data");
	if (teststatus)
		std::cerr << "ERROR: check_raw_results failed." << std::endl;
	return teststatus;
}


bool check_wenni_snr_results(void)
{
	bool teststatus=false;
	teststatus|=compare("wenni_imputed.data","../data/wenni_imputed.data",
											1.1e-11);
	if (teststatus)
		std::cerr << "ERROR: check_wenni_results failed." << std::endl;
	return teststatus;
}


bool cleanup(void)
{
	bool teststatus=false;
	teststatus|=rm("BASEfile.data");
	teststatus|=rm("basefile_out.data");
	teststatus|=rm("knni_imputed.data");
	teststatus|=rm("wenni_BCh1Mean.data");
	teststatus|=rm("wenni_BCh1SD.data");
	teststatus|=rm("wenni_BCh2Mean.data");
	teststatus|=rm("wenni_BCh2SD.data");
	teststatus|=rm("wenni_FCh1Mean.data");
	teststatus|=rm("wenni_FCh2Mean.data");
	teststatus|=rm("wenni_imputed.data");
	teststatus|=rm("wenni_intensity1.data");
	teststatus|=rm("wenni_intensity2.data");
	teststatus|=rm("wenni_logratio.data");
	teststatus|=rm("wenni_weight.data");
	return teststatus;
}


bool compare(const std::string& a,const std::string& b, double error_bound)
{
	bool teststatus=false;
	std::ifstream astream(a.c_str());
	std::ifstream bstream(b.c_str());
	theplu::gslapi::matrix adata(astream);
	theplu::gslapi::matrix bdata(bstream);
	adata-=bdata;
	for (size_t i=0; i<adata.rows(); i++)
		for (size_t j=0; j<adata.columns(); j++)
			if ((fabs(adata(i,j))>error_bound) || // result outside acceptable bounds
					(!((adata(i,j)==adata(i,j)))))    // calucaltion error detected
				teststatus=true;
	astream.close();
	bstream.close();
	return teststatus;
}


bool diff(const std::string& a,const std::string& b)
{
	bool teststatus=false;
	std::string system_call = "diff "+a+" "+b+" >& /dev/null";
	if (system(system_call.c_str())) {
		teststatus|=true;
		std::cerr << "ERROR: system call failed:\n\t" << system_call << std::endl;
	}
	return teststatus;
}


bool rm(const std::string& filename) {
	if (unlink(filename.c_str())) {
		std::cerr << "ERROR: unlink call failed for: " << filename << std::endl;
		return true;
	}
	return false;
}
